@@ -4,6 +4,7 @@
 import base64
 from datetime import datetime, timedelta
 import json
+from re import T
 try:
     import asyncio
 except:
@@ -123,18 +124,33 @@ class pyShelly():
         except:
             pass
 
-        self._send_discovery_timer = timer(timedelta(seconds=60))
-        self._check_by_ip_timer = timer(timedelta(seconds=60))        
+        self._send_discovery_timer = timer(60)
+        self._check_by_ip_timer = timer(10)        
 
-    def send_mqtt(self, block, topic, payload):
+    def send_mqtt(self, block, topic, payload, rpc, params=None):
+        res = False
+        if block.rpc:
+            payload = { "id" : 123, 
+                        "src" : "shelly4hass",
+                        "method" : rpc,
+                        "params" : params }
+            topic = block.mqtt_name + "/rpc"
+        else:
+            topic = "shellies/" + block.mqtt_name + "/" + topic
+            
         if isinstance(payload, dict):
             payload = json.dumps(payload)
         if not isinstance(payload, str):
             payload = str(payload)
+
         if self._mqtt_client and block.mqtt_src=="Client":
-            self._mqtt_client.send(block.mqtt_name, topic, payload)
+            self._mqtt_client.send(block, topic, payload)
+            res = True
         if self._mqtt_server and block.mqtt_src=="Server":
-            self._mqtt_server.send(block.mqtt_name, topic, payload)
+            self._mqtt_server.send(block, topic, payload)
+            res = True
+
+        return res
 
     def load_cache(self, name):
         if self.cb_load_cache:
@@ -198,65 +214,95 @@ class pyShelly():
         if self._coap:
             self._coap.discover()
 
-    def add_device_by_ip(self, ip_addr, src):
-        if ip_addr not in self._shelly_by_ip:
-            LOGGER.debug("Add device by host %s %s", ip_addr, src)
-            self._shelly_by_ip[ip_addr] = {'done':False, 'src':src,
+    def add_device_by_ip(self, id, src): 
+        if not id:
+            return
+        uid = "%s|%s" % (src, id)
+        if uid not in self._shelly_by_ip:
+            LOGGER.debug("Add device by %s %s", id, src)
+            self._shelly_by_ip[uid] = {'done':False, 'src':src, 'id' : id, 
                                            'poll_block':None}
         else:
-            block = self._shelly_by_ip[ip_addr]['poll_block']
+            block = self._shelly_by_ip[uid]['poll_block']
             if block:
                 if not src in block.protocols:
                     block.protocols.append(src)
                 self._poll_block(block)
 
+
     def check_by_ip(self):
-        for ip_addr in list(self._shelly_by_ip.keys()):
-            data = self._shelly_by_ip[ip_addr]
+        for uid in list(self._shelly_by_ip.keys()):
+            data = self._shelly_by_ip[uid]
             delay = data.get('delay')
             if delay:
                 data['delay'] = delay - 1
                 continue
+            if data["src"] == "mDns" and "ip-addr" in data:
+                ip_addr = self._mdns.get_ip(*data['id'].split('|'))
+                if data["ip-addr"] != ip_addr:
+                    data["done"] = False
             if not data['done']:
-                done = False
-                for id, block in self.blocks.items():
+                done = False                
+                if data['src'] == 'mDns':
+                    ip_addr = self._mdns.get_ip(*data['id'].split('|'))
+                else:
+                    ip_addr = data['id']
+                data["ip-addr"] = ip_addr
+                for id in self.blocks.keys():
+                    block = self.blocks[id]                    
                     if block.ip_addr == ip_addr:
-                        self._shelly_by_ip[ip_addr]['done'] = True
-                        done = True
+                        self._shelly_by_ip[uid]['done'] = True
+                        done = True                
                 if done:
                     continue
-                success, settings = shelly_http_get(
-                    ip_addr, "/settings", self.username, self.password, False)
+                success, shelly = shelly_http_get(
+                    ip_addr, "/shelly", self.username, self.password, False)                
                 if success:
-                    success, status = shelly_http_get(
-                        ip_addr, "/status", self.username, self.password, False)
-                    if success:
+                    gen = shelly.get('gen', 1)                    
+                    if gen==1:
+                        success, settings = shelly_http_get(
+                            ip_addr, "/settings", self.username, self.password, False)
+                        if success:
+                            success, status = shelly_http_get(
+                                ip_addr, "/status", self.username, self.password, False)
+                            if success:
+                                done = True
+                                self._shelly_by_ip[uid]['done'] = True
+                                dev = settings["device"]
+                                device_id = dev["hostname"].rpartition('-')[2]
+                                device_type = dev["type"]
+                                ip_addr = status["wifi_sta"]["ip"]
+                                LOGGER.debug("Add device from IP, %s, %s, %s", device_id, device_type, ip_addr)
+                                self.update_block(device_id, device_type, ip_addr,
+                                                data['src'],
+                                                None)
+                                if device_id in self.blocks:
+                                    block = self.blocks[device_id]
+                                    if block and block.sleep_device:
+                                        self._shelly_by_ip[uid]['poll_block'] \
+                                            = block
+                                self._shelly_by_ip[uid]['errors'] = 0
+                    elif gen==2:
                         done = True
-                        self._shelly_by_ip[ip_addr]['done'] = True
-                        dev = settings["device"]
-                        if not "hostname" in dev: #invalide device
-                            LOGGER.info("Error adding device (missing hostname), %s %s",
-                                ip_addr, data['src'])
-                            continue
-                        device_id = dev["hostname"].rpartition('-')[2]
-                        device_type = dev["type"]
-                        ip_addr = status["wifi_sta"]["ip"]
+                        self._shelly_by_ip[uid]['done'] = True
+                        device_id = shelly["mac"]
+                        device_type = "Shelly" + shelly["app"]
                         LOGGER.debug("Add device from IP, %s, %s, %s", device_id, device_type, ip_addr)
                         self.update_block(device_id, device_type, ip_addr,
-                                          data['src'],
-                                          None)
+                                        data['src'],
+                                        None)
                         if device_id in self.blocks:
                             block = self.blocks[device_id]
                             if block and block.sleep_device:
-                                self._shelly_by_ip[ip_addr]['poll_block'] \
+                                self._shelly_by_ip[uid]['poll_block'] \
                                     = block
-                        self._shelly_by_ip[ip_addr]['errors'] = 0
+                        self._shelly_by_ip[uid]['errors'] = 0
                 if not done:
                     errors = data.get('errors', 0)
-                    self._shelly_by_ip[ip_addr]['errors'] = errors + 1
-                    self._shelly_by_ip[ip_addr]['delay'] = 60
+                    self._shelly_by_ip[uid]['errors'] = errors + 1
+                    self._shelly_by_ip[uid]['delay'] = 60
                     if errors > 5:
-                        self._shelly_by_ip[ip_addr]['delay'] = 600
+                        self._shelly_by_ip[uid]['delay'] = 600
                     LOGGER.info("Error adding device, %s %s",
                                 ip_addr, data['src'])
 
@@ -281,6 +327,8 @@ class pyShelly():
 
     def update_block(self, block_id, device_type, ipaddr, src, payload,
                      force_poll=False, mqtt=None):
+        block_id = block_id.upper()
+
         if self.only_device_id is not None and \
             block_id != self.only_device_id:
             return
@@ -298,6 +346,10 @@ class pyShelly():
         if not src in block.protocols:
             block.protocols.append(src)
 
+        if ipaddr and block.ip_addr != ipaddr:
+            block.ip_addr = ipaddr
+            block.force_all_update()
+
         if payload:
             if src == "MQTT":
                 block.update_mqtt(payload)
@@ -309,8 +361,6 @@ class pyShelly():
         if block_added:
             for callback in self.cb_block_added:
                 callback(block)
-            #for device in block.devices:
-            #    self.add_device(device, block.discovery_src)
 
         if block.sleep_device or force_poll:
             self._poll_block(block, force_poll)
@@ -334,7 +384,7 @@ class pyShelly():
                     block.check_available()
                     block.loop()
 
-                self.stopped.wait(5)
+                self.stopped.wait(1)
             except Exception as ex:
                 exception_log(ex, "Error update loop")
 
@@ -349,6 +399,6 @@ class pyShelly():
             block.last_update_status_info = now
             t = threading.Thread(
                 target=block.update_status_information)
-            t.name = "Poll status"
+            t.name = "S4H-Poll status"
             t.daemon = True
             t.start()
