@@ -10,11 +10,14 @@ from .const import (
     ATTR_POS,
     ATTR_CHANNEL,
     ATTR_TOPIC,
+    ATTR_RPC,
     REGEX_VER,
     SRC_COAP,
     SRC_STATUS,
     SRC_MQTT,
-    SRC_MQTT_STATUS
+    SRC_MQTT_STATUS,
+    SRC_WS,
+    SRC_WS_STATUS
 )
 class Base(object):
 
@@ -26,11 +29,15 @@ class Base(object):
         self.info_values_coap = {}
         self.info_values_mqtt = {}
         self.info_values_mqtt_status = {}
+        self.info_values_ws = {}
+        self.info_values_ws_status = {}
         self._state_cfg = None
         self.state_status = None
         self.state_coap = None
         self.state_mqtt = None
         self.state_mqtt_status = None
+        self.state_ws = None
+        self.state_ws_status = None
         self._channel = 0
         self.cb_updated = []
         self.need_update = False
@@ -150,6 +157,10 @@ class Base(object):
                 self.state_mqtt = new_state
             if src == SRC_MQTT_STATUS:
                 self.state_mqtt_status = new_state
+            if src == SRC_WS:
+                self.state_ws = new_state
+            if src == SRC_WS_STATUS:
+                self.state_ws_status = new_state
             if self.debug:
                 self.need_update = True
             if self.state != new_state:
@@ -169,9 +180,9 @@ class Base(object):
             self.__update_info_values_coap(payload, self._info_value_cfg)
 
     def _get_mqtt_value(self, cfg, payload):
+        payload_topic = payload['topic']            
         if ATTR_TOPIC in cfg:
             topic_list = cfg[ATTR_TOPIC]
-            payload_topic = payload['topic']            
             if not type(topic_list) is list:
                 topic_list = [topic_list]
             for topic in topic_list:
@@ -185,7 +196,7 @@ class Base(object):
                     if payload_topic == topic:         
                         return self._fmt_info_value(payload['data'], cfg, SRC_MQTT)                
 
-    def _update_info_values_mqtt(self, payload, extra_info_value_cfg=None):
+    def _update_info_values_mqtt(self, payload, extra_info_value_cfg=None):                
         if self._state_cfg:
             new_state = self._get_mqtt_value(self._state_cfg, payload)
             self._set_state(new_state, SRC_MQTT)
@@ -198,6 +209,36 @@ class Base(object):
                 value = self._get_mqtt_value(cfg, payload)
                 self.set_info_value(name, value, SRC_MQTT)
 
+    def _get_rpc_value(self, cfg, rpc_data):
+        if ATTR_RPC in cfg:            
+            rpc_list = cfg[ATTR_RPC]
+            if not type(rpc_list) is list:
+                rpc_list = [rpc_list]
+            for rpc in rpc_list:
+                rpc = rpc.replace('$', str(cfg.get(ATTR_CHANNEL, self._channel))) #Todo cache??
+                value = rpc_data 
+                for key in rpc.split('/'):
+                    if value is not None:
+                        if key.startswith('include:'):
+                            value = (key[8:] in value)
+                        else:
+                            value = value.get(key, None)
+                if value != None:
+                    return self._fmt_info_value(value, cfg, SRC_MQTT)  
+
+    def _update_info_values_rpc(self, rpc_data, src, extra_info_value_cfg=None):                
+        if self._state_cfg:
+            new_state = self._get_rpc_value(self._state_cfg, rpc_data)
+            self._set_state(new_state, src)
+        if extra_info_value_cfg:
+            for name, cfg in extra_info_value_cfg.items():
+                value = self._get_rpc_value(cfg, rpc_data)
+                self.set_info_value(name, value, src)                  
+        if self._info_value_cfg:
+            for name, cfg in self._info_value_cfg.items():
+                value = self._get_rpc_value(cfg, rpc_data)
+                self.set_info_value(name, value, src)
+
     #Todo: remove
     def coap_get(self, payload, pos_list, default=None, channel=None):
         if pos_list is None:
@@ -229,6 +270,10 @@ class Base(object):
             self.info_values_mqtt[name] = value
         elif src == SRC_MQTT_STATUS:
             self.info_values_mqtt_status[name] = value
-        if self.debug:
-            self.need_update = True
+        elif src == SRC_WS:
+            self.info_values_ws[name] = value
+        elif src == SRC_WS_STATUS:
+            self.info_values_ws_status[name] = value
+        #if self.debug:
+        #    self.need_update = True
 
