@@ -1,8 +1,10 @@
 """Block is a physical device"""
 # pylint: disable=broad-except, bare-except
 
+from distutils.log import debug
 import json
-from datetime import datetime, timedelta
+from datetime import datetime
+
 from .utils import shelly_http_get, timer
 from .switch import Switch
 from .relay import Relay
@@ -14,7 +16,7 @@ from .dimmer import Dimmer
 from .roller import Roller
 from .utils import exception_log
 from .base import Base
-
+from .ws_client import WebSocket
 
 from .const import (
     LOGGER,
@@ -43,6 +45,7 @@ from .const import (
     ATTR_FMT,
     ATTR_POS,
     ATTR_AUTO_SET,
+    ATTR_RPC,
     BLOCK_INFO_VALUES,
     SHELLY_TYPES,
     SRC_STATUS,
@@ -57,6 +60,7 @@ class Block(Base):
         self.id = block_id
         self.unit_id = block_id
         self.type = block_type
+        self.rpc = False
         self.parent = parent
         self.ip_addr = ip_addr
         self.devices = []
@@ -75,8 +79,10 @@ class Block(Base):
         self.payload = None
         self.settings = None
         self.exclude_info_values = []
+        self.websocket = None
         #self._info_value_cfg = None
         self._need_setup_delayed_devices = False
+        self._cnt_setup_delayed_devices = 0
         self.setup_devices()
         self._available = None
         self.status_update_error_cnt = 0
@@ -84,7 +90,7 @@ class Block(Base):
         self._last_friendly_name = None
         self.mqtt_name = None
         self.mqtt_src = None        
-        self._check_delay_load = timer(timedelta(seconds=60))
+        self._check_delay_load = timer(1)
 
     def update_coap(self, payload, ip_addr):
         self.ip_addr = ip_addr  # If changed ip
@@ -109,35 +115,67 @@ class Block(Base):
         #     self.reload = False
         self.raise_updated()
 
+    def update_rpc(self, rpc_data, src):
+        self.last_updated = datetime.now()
+        ipaddr = self._get_rpc_value({ATTR_RPC:'wifi/sta_ip'}, rpc_data)
+        if ipaddr:
+            self.ip_addr=ipaddr
+        self._update_info_values_rpc(rpc_data, src, BLOCK_INFO_VALUES)
+        if 'events' in rpc_data:
+            events = rpc_data['events']
+            for event in events:
+                comp = event.get("component")
+                type = event.get("event")
+                for dev in self.devices:
+                    if hasattr(dev, 'rpc_event'):
+                        dev.rpc_event(comp, type)
+
+        self._update_infovalues(src)
+
+        force_update_devices = self.need_update #Block updated
+        self.raise_updated()
+        
+        for dev in self.devices:
+            dev._update_info_values_rpc(rpc_data, src)
+            if hasattr(dev, 'update_rpc'):
+                dev.update_rpc(rpc_data, src)
+            dev.raise_updated(force_update_devices)
+        self.raise_updated()
+
     def update_mqtt(self, payload):
         self.mqtt_name = payload['name']
         self.mqtt_src = payload['src']
         self.last_updated = datetime.now()
-
-        if payload['topic'] == "info":
+        topic = payload['topic']
+        if topic == "info":
             status = json.loads(payload['data'])
             self._update_status_info(status, SRC_MQTT_STATUS)
-        elif payload['topic'] == "announce":
+        elif topic == "announce":
             pass
         else:
-            self._update_info_values_mqtt(payload, BLOCK_INFO_VALUES)
-            for dev in self.devices:
-                dev._update_info_values_mqtt(payload)
-                if hasattr(dev, 'update_mqtt'):
-                    dev.update_mqtt(payload)
-                dev.raise_updated()
-            # if self.reload:
-            #     self._reload_devices()
-            #     for dev in self.devices:
-            #         dev._update_info_values_coap(payload)
-            #         if hasattr(dev, 'update_coap'):
-            #             dev.update_coap(payload)
-            #         self.parent.add_device(dev, self.discovery_src)
-            #     self.reload = False
-            self.raise_updated()
+            if self.rpc:
+                data = json.loads(payload['data'])
+                data = data['params'] if 'params' in data else data['result']
+                self.update_rpc(data, SRC_MQTT)
+            else:
+                self._update_info_values_mqtt(payload, BLOCK_INFO_VALUES)
+                for dev in self.devices:
+                    dev._update_info_values_mqtt(payload)
+                    if hasattr(dev, 'update_mqtt'):
+                        dev.update_mqtt(payload)
+                    dev.raise_updated()
+                self.raise_updated()
 
     def loop(self):
-        if self._need_setup_delayed_devices and self._check_delay_load.check():            
+        if self.websocket:
+            self.websocket.check()
+
+        if self._need_setup_delayed_devices and self._check_delay_load.check():
+            self._cnt_setup_delayed_devices += 1
+            if self._cnt_setup_delayed_devices == 10:
+                self._check_delay_load.set_interval(60)
+            if self._cnt_setup_delayed_devices == 60:
+                self._check_delay_load.set_interval(3600)
             if self.setup_devices_delayed():
                 self._need_setup_delayed_devices = False
 
@@ -156,19 +194,28 @@ class Block(Base):
                                 self.set_info_value(iv, new_value, None)
                                 self.raise_updated()
 
+    def force_all_update(self):
+        self.raise_updated(True)
+        for dev in self.devices:
+            dev.raise_updated(True)
+
     def check_available(self):
         if self.available() != self._available:
             self._available = self.available()
-            self.raise_updated(True)
-            for dev in self.devices:
-                dev.raise_updated(True)
+            self.force_all_update()
 
     def update_status_information(self):
         """Update the status information."""
         self.last_update_status_info = datetime.now()
         if self.mqtt_name:
-            self.parent.send_mqtt(self, "command", "announce")
-        if self.ip_addr:
+            self.parent.send_mqtt(self, "command", "announce",
+                "Shelly.GetStatus")
+        res = False
+        if self.websocket:
+            res = self.websocket.send("Shelly.GetStatus")
+            self.websocket.send("Shelly.GetConfig")
+
+        if not res and self.ip_addr:
             if self.status_update_error_cnt >= 3:
                 diff = (datetime.now()-self.last_try_update_status).total_seconds()
                 if diff < 600 or (diff < 3600 and self.status_update_error_cnt >= 5):
@@ -177,7 +224,8 @@ class Block(Base):
             self.last_try_update_status = datetime.now()
 
             LOGGER.debug("Get status from %s %s", self.id, self.friendly_name())
-            success, status = self.http_get('/status', False)
+            url = "/rpc/Shelly.GetStatus" if self.rpc else "/status"
+            success, status = self.http_get(url, False)
 
             if not success or status == {}:
                 self.status_update_error_cnt += 1
@@ -188,7 +236,42 @@ class Block(Base):
 
             self.status_update_error_cnt = 0
 
-            self._update_status_info(status, SRC_STATUS)
+            if self.rpc:
+                self.update_rpc(status, SRC_STATUS)
+            else:
+                self._update_status_info(status, SRC_STATUS)
+
+    def _update_infovalues(self, src): 
+        cloud_status = ''
+        connected = self.info_values.get(INFO_VALUE_CLOUD_CONNECTED)
+        enabled = self.info_values.get(INFO_VALUE_CLOUD_ENABLED)
+        if connected:
+            cloud_status = 'connected'
+        elif enabled:
+            cloud_status = 'disconnected'
+        elif enabled is not None:
+            cloud_status = 'disabled'
+
+        if cloud_status:
+            self.set_info_value(INFO_VALUE_CLOUD_STATUS, cloud_status, src)
+
+        rssi = self.info_values.get(INFO_VALUE_RSSI)
+        rssi_level = self.info_values.get(INFO_VALUE_RSSI_LEVEL)
+        if rssi is not None:
+            for val, name in RSSI_LEVELS.items():
+                if rssi >= val or ( rssi_level == name and rssi >= val - 2):
+                    rssi_level = name
+                    break
+            self.set_info_value(INFO_VALUE_RSSI_LEVEL, rssi_level, src)
+
+        #self.info_values[INFO_VALUE_HAS_FIRMWARE_UPDATE] = self.has_fw_update()
+        self.set_info_value(INFO_VALUE_LATEST_BETA_FW_VERSION,
+                            self.latest_fw_version(True), src)
+
+        friendly_name = self.friendly_name()
+        if self._last_friendly_name!=friendly_name:
+            self._last_friendly_name=friendly_name
+            self.need_update=True
 
     def _update_status_info(self, status, src):
         self.last_updated = datetime.now()
@@ -208,31 +291,8 @@ class Block(Base):
             for name, cfg in self._info_value_cfg.items():
                 self._update_info_value(name, status, cfg, src)
 
-        cloud_status = 'disabled'
-        if self.info_values.get(INFO_VALUE_CLOUD_ENABLED):
-            if self.info_values.get(INFO_VALUE_CLOUD_CONNECTED):
-                cloud_status = 'connected'
-            else:
-                cloud_status = 'disconnected'
-        self.set_info_value(INFO_VALUE_CLOUD_STATUS, cloud_status, src)
-
-        rssi = self.info_values.get(INFO_VALUE_RSSI)
-        rssi_level = self.info_values.get(INFO_VALUE_RSSI_LEVEL)
-        for val, name in RSSI_LEVELS.items():
-            if rssi >= val or ( rssi_level == name and rssi >= val - 2):
-                rssi_level = name
-                break
-        self.set_info_value(INFO_VALUE_RSSI_LEVEL, rssi_level, src)
-
-        #self.info_values[INFO_VALUE_HAS_FIRMWARE_UPDATE] = self.has_fw_update()
-        self.set_info_value(INFO_VALUE_LATEST_BETA_FW_VERSION,
-                            self.latest_fw_version(True), src)
-
-        friendly_name = self.friendly_name()
-        if self._last_friendly_name!=friendly_name:
-            self._last_friendly_name!=friendly_name
-            self.need_update
-
+        self._update_infovalues(src)
+        
         force_update_devices = self.need_update #Block updated
         self.raise_updated()
 
@@ -330,7 +390,7 @@ class Block(Base):
             self._add_device(Relay(self, 0))
             self._add_device(Switch(self, 1))
             self._add_device(Switch(self, 2))
-        #Shelly 1 PM
+        #Shelly 1PM
         elif self.type == 'SHSW-PM':
             self._add_device(Relay(self, 0))
             self._add_device(PowerMeter(self, 0))
@@ -340,8 +400,19 @@ class Block(Base):
             self._add_device(ExtTemp(self, 2), True)
             self._add_device(ExtHumidity(self, 0), True)
             self._add_device(ExtSwitch(self), True)
+        #Shelly Plus 1
+        elif self.type == 'ShellyPlus1':
+            self.rpc = True
+            self._add_device(Relay(self, 0))
+            self._add_device(Switch(self, 0))
+        #Shelly Plus 1PM
+        elif self.type == 'ShellyPlus1PM':
+            self.rpc = True
+            self._add_device(Relay(self, 0))
+            self._add_device(PowerMeter(self, 0, gen=2))
+            self._add_device(Switch(self, 0))
         #Shelly 2
-        elif self.type == 'SHSW-21':            
+        elif self.type == 'SHSW-21':           
             self._add_device(Switch(self, 1))
             self._add_device(Switch(self, 2))
             self._add_device(PowerMeter(self, 0))
@@ -378,6 +449,39 @@ class Block(Base):
                 self._add_device(Relay(self, channel + 1))
                 self._add_device(PowerMeter(self, channel + 1))
                 self._add_device(Switch(self, channel + 1))
+        #Shelly Pro 1
+        elif self.type == 'ShellyPro1':
+            self.rpc = True
+            self._add_device(Relay(self, 1))
+            self._add_device(Switch(self, 1))
+            self._add_device(Switch(self, 2))
+        #Shelly Pro 1PM
+        elif self.type == 'ShellyPro1PM':
+            self.rpc = True
+            self._add_device(Relay(self, 1))
+            self._add_device(PowerMeter(self, 1, gen=2))
+            self._add_device(Switch(self, 1))
+            self._add_device(Switch(self, 2))
+        #Shelly Pro 2
+        elif self.type == 'ShellyPro2':
+            self.rpc = True
+            for channel in range(2):
+                self._add_device(Relay(self, channel + 1))
+                self._add_device(Switch(self, channel + 1))
+        #Shelly Pro 2PM
+        elif self.type == 'ShellyPro2PM':
+            self.rpc = True
+            for channel in range(2):
+                self._add_device(Relay(self, channel + 1))
+                self._add_device(PowerMeter(self, channel + 1, gen=2))
+                self._add_device(Switch(self, channel + 1))
+        #Shelly Pro 4PM
+        elif self.type == 'ShellyPro4PM':
+            self.rpc = True
+            for channel in range(4):
+                self._add_device(Relay(self, channel + 1))
+                self._add_device(PowerMeter(self, channel + 1, gen=2))
+                self._add_device(Switch(self, channel + 1))
         elif self.type == 'SHRGBWW-01':
             self._add_device(RGBWW(self))
         #Shelly Dimmer
@@ -394,6 +498,14 @@ class Block(Base):
             self.exclude_info_values.append(INFO_VALUE_DEVICE_TEMP)
             self._add_device(TempSensor(self))
             self._add_device(Sensor(self, [44, 3103], 'humidity', 'hum/value', topic= 'sensor/humidity'))
+        #Shelly H&T Plus
+        elif self.type == 'SNSN-0013A':
+            debug
+            self.sleep_device = True
+            self.unavailable_after_sec = SENSOR_UNAVAILABLE_SEC
+            self.exclude_info_values.append(INFO_VALUE_DEVICE_TEMP)
+            self._add_device(TempSensor(self))
+            self._add_device(Sensor(self, [44, 3103], 'humidity', 'hum/value', topic= 'sensor/humidity'))
         #Shellyy RGBW2
         elif self.type == 'SHRGBW2':
             self._add_device(Switch(self, 0))
@@ -445,10 +557,18 @@ class Block(Base):
             self.unavailable_after_sec = SENSOR_UNAVAILABLE_SEC
             self._add_device(Switch(self, 0, simulate_state=True,
                                     master_unit=True))
+        #Shelly i3
         elif self.type == 'SHIX3-1':
             self._add_device(Switch(self, 1, master_unit=True))
             self._add_device(Switch(self, 2))
             self._add_device(Switch(self, 3))
+        #Shelly Plus i4
+        elif self.type == 'ShellyPlusI4':
+            self.rpc = True
+            self._add_device(Switch(self, 1, master_unit=True))
+            self._add_device(Switch(self, 2))
+            self._add_device(Switch(self, 3))
+            self._add_device(Switch(self, 4))
         elif self.type == 'SHGS-1':
             self._info_value_cfg = {INFO_VALUE_GAS : {ATTR_POS : 122},
                                     INFO_VALUE_SENSOR : {ATTR_POS : [118, 2101]}
@@ -481,7 +601,10 @@ class Block(Base):
             }
             #shellies/shellymotionsensor-60A423976594/status {"motion":true,"timestamp":1614416952,"active":true,"vibration":true,"lux":303,"bat":87}
             #{"G":[[0,6107,1],[0,3119,1614417090],[0,3120,1],[0,6110,0],[0,3106,285],[0,3111,87],[0,9103,11]]}
-            self._add_device(Motion(self))
+            self._add_device(Motion(self))    
+
+        if (self.rpc):
+            self.websocket = WebSocket(self)
 
     def _add_device(self, dev, lazy_load=False, major=False):
         dev.lazy_load = lazy_load
