@@ -19,7 +19,7 @@ class CoAP():
     def __init__(self, root):
         self._root = root
         self._thread = threading.Thread(target=self._loop)
-        self._thread.name = "CoAP"
+        self._thread.name = "S4H-CoAP"
         self._thread.daemon = True
         self._socket = None
 
@@ -193,5 +193,5 @@ class CoAP():
 
             except Exception as ex:
                 #LOGGER.debug("Error receive CoAP %s", str(ex))
-                LOGGER.exception("Error receive CoAP")
-                #exception_log(ex, "Error receiving CoAP UDP")
+                #LOGGER.exception("Error receive CoAP,  " + str(ex))
+                exception_log(ex, "Error receiving CoAP UDP")
