@@ -5,7 +5,7 @@ import logging
 LOGGER = logging.getLogger('pyShelly')
 
 NAME = "pyShelly"
-VERSION = "0.2.23"
+VERSION = "1.0.0"
 
 COAP_IP = "224.0.1.187"
 COAP_PORT = 5683
@@ -13,7 +13,9 @@ COAP_PORT = 5683
 MDNS_IP = "224.0.0.251"
 MDNS_PORT = 5353
 
-REGEX_VER = r"^20(\d{6}).+v(\d+\.\d+\.\d+(-rc\d)?)(_.*)?@"
+#20210323-105928/v1.10.1-gf276b51
+#20201019-110845/v1.6.6_4Pro@bad181e4
+REGEX_VER = r"^20(\d{6}).+v(\d+\.\d+\.\d+(-rc\d)?)([^\d].*)?$"
 
 """Define constants for result from /status response from device"""
 STATUS_RESPONSE_RELAYS = 'relays'
@@ -58,6 +60,7 @@ INFO_VALUE_RSSI = 'rssi'
 INFO_VALUE_RSSI_LEVEL = 'rssi_level'
 INFO_VALUE_UPTIME = 'uptime'
 INFO_VALUE_OVER_POWER = 'over_power'
+INFO_VALUE_OVER_VOLTAGE = 'over_voltage'
 INFO_VALUE_DEVICE_TEMP = 'device_temp'
 INFO_VALUE_OVER_TEMPERATURE = 'over_temp'
 INFO_VALUE_SSID = 'ssid'
@@ -93,28 +96,32 @@ ATTR_POS = 'pos'
 ATTR_CHANNEL = 'channel'
 ATTR_AUTO_SET = 'auto_set'
 ATTR_TOPIC = 'topic'
+ATTR_RPC = 'rpc'
 
 SRC_COAP = 1
 SRC_STATUS = 2
 SRC_MQTT = 3
 SRC_MQTT_STATUS = 4
+SRC_RPC = 5
+SRC_WS = 6
+SRC_WS_STATUS = 7
 
 BLOCK_INFO_VALUES = {
-    INFO_VALUE_SSID : {ATTR_PATH :'wifi_sta/ssid'},
-    INFO_VALUE_RSSI : {ATTR_PATH : 'wifi_sta/rssi'},
-    INFO_VALUE_UPTIME : {ATTR_PATH : 'uptime'},
-    INFO_VALUE_DEVICE_TEMP : {ATTR_PATH : 'tmp/tC', ATTR_POS: [311, 3104], ATTR_TOPIC: 'temperature'}, #, ATTR_FMT : 'round'},
-    INFO_VALUE_OVER_TEMPERATURE : {ATTR_PATH : 'overtemperature', ATTR_POS: 6101, ATTR_TOPIC: 'overtemperature'},
+    INFO_VALUE_SSID : {ATTR_PATH :'wifi_sta/ssid', ATTR_RPC:'wifi:ssid'},
+    INFO_VALUE_RSSI : {ATTR_PATH : 'wifi_sta/rssi', ATTR_RPC:'wifi/rssi'},
+    INFO_VALUE_UPTIME : {ATTR_PATH : 'uptime', ATTR_RPC:'sys/uptime'},
+    INFO_VALUE_DEVICE_TEMP : {ATTR_PATH : 'tmp/tC', ATTR_POS: [311, 3104], ATTR_TOPIC: 'temperature', ATTR_RPC:'switch:$/temperature/tC'}, #, ATTR_FMT : 'round'},
+    INFO_VALUE_OVER_TEMPERATURE : {ATTR_PATH : 'overtemperature', ATTR_POS: 6101, ATTR_TOPIC: 'overtemperature', ATTR_RPC: 'switch:$/errors/include:overtemp'},
     INFO_VALUE_HAS_FIRMWARE_UPDATE : {ATTR_PATH : 'update/has_update'},
     INFO_VALUE_LATEST_FIRMWARE_VERSION : {ATTR_PATH : 'update/new_version',
                                           ATTR_FMT : 'ver'},
     INFO_VALUE_FW_VERSION : {ATTR_PATH : 'update/old_version',
                              ATTR_FMT : 'ver'},
-    INFO_VALUE_CLOUD_ENABLED : {ATTR_PATH : 'cloud/enabled'},
-    INFO_VALUE_CLOUD_CONNECTED : {ATTR_PATH : 'cloud/connected'},
-    INFO_VALUE_MQTT_CONNECTED : {ATTR_PATH : 'mqtt/connected'},
+    INFO_VALUE_CLOUD_ENABLED : {ATTR_PATH : 'cloud/enabled', ATTR_RPC : 'cloud/enable'}, #ed remover 220428
+    INFO_VALUE_CLOUD_CONNECTED : {ATTR_PATH : 'cloud/connected', ATTR_RPC : 'cloud/connected'},
+    INFO_VALUE_MQTT_CONNECTED : {ATTR_PATH : 'mqtt/connected', ATTR_RPC : 'mqtt/connected'},
     #INFO_VALUE_CURRENT_CONSUMPTION : {ATTR_PATH : 'consumption'},
-    INFO_VALUE_VOLTAGE : {ATTR_PATH : 'voltage', ATTR_FMT : 'round'},
+    INFO_VALUE_VOLTAGE : {ATTR_PATH : 'voltage', ATTR_FMT : 'round', ATTR_RPC : 'switch:$/voltage'},
     INFO_VALUE_BATTERY : {ATTR_PATH : 'bat/value', ATTR_POS : [77, 3111], ATTR_TOPIC: "sensor/battery"},
     #INFO_VALUE_TILT : {ATTR_PATH : 'accel/tilt'}, #Todo
     #INFO_VALUE_VIBRATION : {ATTR_PATH : 'accel/vibration'}, #Todo
@@ -153,7 +160,7 @@ SHELLY_TYPES = {
     'SHDW-2': {'name': "Shelly Door/Window 2", 'battery' : True,
                'mqtt':'shellydw2'},
     'SHBDUO-1': {'name': "Shelly Duo", 'mqtt':'ShellyBulbDuo'},
-    'SHBVIN-1': {'name': "Shelly Vintage", 'mqtt':'ShellyVintage'},
+    'SHVIN-1': {'name': "Shelly Vintage", 'mqtt':'ShellyVintage'},
     'SHBTN-1': {'name': "Shelly Button 1", 'mqtt':'shellybutton1', 'battery' : True},
     'SHBTN-2': {'name': "Shelly Button 1", 'battery' : True},
     'SHIX3-1': {'name': "Shelly i3", 'mqtt':'shellyix3'},
@@ -162,7 +169,16 @@ SHELLY_TYPES = {
     'SHPLG-U1': {'name': "Shelly Plug US", 'mqtt':'shellyplugu1'},
     'SHUNI-1': {'name': "Shelly UNI", 'mqtt':'shellyuni'},
     'SHSW-L': {'name': "Shelly 1L", 'mqtt':'shelly1l'},
-    'SHMOS-01': {'name': "Shelly Motion", 'mqtt':'shellymotionsensor'}
+    'SHMOS-01': {'name': "Shelly Motion", 'mqtt':'shellymotionsensor'},
+    'ShellyPlus1': {'name':"Shelly Plus 1", 'mqtt':'shellyplus1'},
+    'ShellyPlus1PM': {'name':"Shelly Plus 1PM", 'mqtt':'shellyplus1pm'},    
+    'ShellyPro1': {'name':"Shelly Pro 1", 'mqtt':'shellypro1'},
+    'ShellyPro1PM': {'name':"Shelly Pro 1PM", 'mqtt':'shellypro1pm'},
+    'ShellyPro2': {'name':"Shelly Pro 2", 'mqtt':'shellypro2'},
+    'ShellyPro2PM': {'name':"Shelly Pro 2PM", 'mqtt':'shellypro2pm'},
+    'ShellyPro4PM': {'name':"Shelly Pro 4PM", 'mqtt':'shellypro4pm'},
+    'ShellyPlusI4': {'name':"Shelly Plus i4", 'mqtt':'shellyplusi4'},
+    'SNSN-0013A': {'name': "Shelly Plus H&T", 'mqtt':'shellyplusht'}
 }
 
 EFFECTS_RGBW2 = [
@@ -188,3 +204,4 @@ RSSI_LEVELS = {
     -70 : "good",
     -1000: "weak"
 }
+
