@@ -28,36 +28,27 @@ class Device(Base):
     def ip_addr(self):
         return self.block.ip_addr
 
-    def friendly_name(self):
+    def cloud_name(self):
+        name = None
         try:
             if self.block.parent.cloud:
                 device_id = self.id.lower().split('-')
-                name = None
                 #add_nr = False
                 idx = int(device_id[1]) if len(device_id) > 1 else 0
                 name = self.block.parent.cloud.get_device_name(device_id[0],
                                                                idx,
                                                                self.ext_sensor)
-
-                # if len(device_id) > 1 and int(device_id[1]) > 1:
-                #     cloud_id = device_id[0] + '_' + str(int(device_id[1])-1)
-                #     name = self.block.parent.cloud.get_device_name(cloud_id,
-                #                                                self.ext_sensor)
-                #     if not name:
-                #         add_nr = True
-                # if not name:
-                #     name = \
-                #       self.block.parent.cloud.get_device_name(device_id[0],
-                #                                               self.ext_sensor)
-                #     if add_nr:
-                #         name += " - " + device_id[1]
-                if name:
-                    return name
         except Exception as ex:
             LOGGER.debug("Error look up name, %s", ex)
-        name = self.type_name() + ' - ' + self.id
-        #if self.device_nr:
-        #   name += ' - ' + str(self.device_nr)
+        return name
+
+    def device_name(self):
+        return self.type_name() + ' - ' + self.id
+
+    def friendly_name(self):
+        name = self.cloud_name()
+        if not name:
+            name = self.device_name()
         return name
 
     def room_name(self):
@@ -81,11 +72,14 @@ class Device(Base):
             name = name + " (" + self.sub_name + ")"
         return name
 
-    def _send_command(self, url=None, topic=None, payload=None):
-        if self.ip_addr and url:
-            self.block.http_get(url)
-        if topic and self.block.mqtt_name:
-            self.block.parent.send_mqtt(self.block, topic, payload)
+    def _send_command(self, url=None, topic=None, payload=None, rpc_method=None, rpc_params=None):
+        res = False
+        if rpc_method and self.block.websocket:
+            res = self.block.websocket.send(rpc_method, rpc_params)
+        if not res and topic and self.block.mqtt_name:
+            res = self.block.parent.send_mqtt(self.block, topic, payload, rpc_method, rpc_params)
+        if not res and self.ip_addr and url:
+            res, _ = self.block.http_get(url)
         self.block.update_status_interval = None #Force update
 
     def available(self):
