@@ -4,20 +4,12 @@ from .device import Device
 from .utils import notNone
 from .const import (
     #LOGGER,
-    STATUS_RESPONSE_METERS,
-    STATUS_RESPONSE_EMETERS,
-    STATUS_RESPONSE_METERS_POWER,
-    STATUS_RESPONSE_METERS_TOTAL,
-    STATUS_RESPONSE_METERS_VOLTAGE,
-    STATUS_RESPONSE_METERS_PF,
-    STATUS_RESPONSE_METERS_CURRENT,
-    STATUS_RESPONSE_METERS_TOTAL_RETURNED,
-    INFO_VALUE_CURRENT_CONSUMPTION,
+    ATTR_RPC,
     INFO_VALUE_TOTAL_CONSUMPTION,
     INFO_VALUE_TOTAL_RETURNED,
-    INFO_VALUE_VOLTAGE,
     INFO_VALUE_POWER_FACTOR,
     INFO_VALUE_CURRENT,
+    INFO_VALUE_VOLTAGE,
     ATTR_POS,
     ATTR_PATH,
     ATTR_FMT,
@@ -27,7 +19,7 @@ from .const import (
 class PowerMeter(Device):
     """Class to represent a power meter value"""
     def __init__(self, block, channel, position = None,
-                 tot_pos = None, voltage_to_block=False, em=False, topic="emeter"):
+                 tot_pos = None, voltage_to_block=False, em=False, topic="emeter", gen=1):
         #Todo: voltage_to_block
         super(PowerMeter, self).__init__(block)
         self.id = block.id
@@ -53,9 +45,10 @@ class PowerMeter(Device):
             ATTR_POS: notNone(position, [111, 4101, 4102, 4105]),
             ATTR_PATH: meters + '/$/power',
             ATTR_FMT: ['float'],
-            ATTR_TOPIC: ['relay/$/power', topic + '/$/power']
+            ATTR_TOPIC: ['relay/$/power', topic + '/$/power'],
+            ATTR_RPC: 'switch:$/apower'
         }
-        divider = None if em else '/60'
+        divider = None if (em or gen == 2) else '/60'
         self._info_value_cfg = {
             INFO_VALUE_POWER_FACTOR : {
                 ATTR_POS: [114, 4110],
@@ -67,13 +60,15 @@ class PowerMeter(Device):
                 ATTR_POS: [115, 4109],
                 ATTR_PATH: meters + '/$/current',
                 ATTR_FMT: ['float'],
-                ATTR_TOPIC: topic + '/$/current'
+                ATTR_TOPIC: topic + '/$/current',
+                ATTR_RPC: 'switch:$/current'
             },
             INFO_VALUE_TOTAL_CONSUMPTION : {
                 ATTR_POS: tot_pos or [4103, 4104, 4106],
                 ATTR_PATH: meters + '/$/total',
                 ATTR_FMT: ['float', divider, 'round:2'],
-                ATTR_TOPIC: ['relay/$/energy', topic + '/$/total', topic + '/$/energy']
+                ATTR_TOPIC: ['relay/$/energy', topic + '/$/total', topic + '/$/energy'],
+                ATTR_RPC: 'switch:$/aenergy/total'
             },
             INFO_VALUE_TOTAL_RETURNED :
             {
