@@ -7,6 +7,7 @@ import json
 import socket
 from datetime import datetime
 from .compat import s
+from datetime import timedelta
 try:
     import http.client as httplib
 except:
@@ -18,8 +19,12 @@ from .const import (
 
 class timer():
     def __init__(self, interval):
-        self._interval = interval
+        self.set_interval(interval)
         self._last_time = None
+    def set_interval(self, interval):
+        if isinstance(interval, int):
+            interval = timedelta(seconds=interval)
+        self._interval = interval
     def check(self):
         if self._interval is not None:
             now = datetime.now()
@@ -29,6 +34,14 @@ class timer():
                 return True
         return False
 
+def error_log(_msg, *args):
+    msg = _msg.format(*args)
+    LOGGER.error(msg)
+
+def warning_log(_msg, *args):
+    msg = _msg.format(*args)
+    LOGGER.warning(msg)
+
 def exception_log(ex, _msg, *args):
     """Log exception"""
     msg = _msg.format(*args)
